function [RMSE_total, RMSE_10, RMSE_20] = compute_L2_SAR_error(params,evalFile)

% Imports SAR values computed by the numerical code in the grid points.
% Then computes relative L2 errors compared to the Mie solution. The 
% normalization factor is the L2 average of the Mie solution.

% Reads file containing SAR values computed by the numerical code.
% new read function
%evalData = import_SAR_from_txt(evalFile);
% backward compatible file read
evalDataMat = dlmread(evalFile,'\t');
%evalData.Properties.VariableNames = {'I','J','K','x','y','z','SAR'};
evalData.I = evalDataMat(:,1); evalData.J = evalDataMat(:,2); evalData.K = evalDataMat(:,3);
evalData.x = evalDataMat(:,4); evalData.y = evalDataMat(:,5); evalData.z = evalDataMat(:,6);
evalData.SAR = evalDataMat(:,7);

SAR_eval = evalData.SAR;
SAR_error = zeros(size(SAR_eval));
SAR_mie_vec = SAR_error;


% compute coefficients
result = Mie_cd(params.x0,params.m, params);
c=result(1,:);
d=result(2,:);

%% init progress bar
wb = waitbar(0,'Comparing point SAR');

%%

resolution = params.resolution;
x_vec = params.x_grid;
y_vec = params.y_grid;
z_vec = params.z_grid;

%

num_grid_points = length(x_vec)*length(y_vec)*length(z_vec);
current_grid_point = 1;
IJK = zeros(num_grid_points,3);
XYZ = zeros(num_grid_points,3);


% step x,y,z coordinates
for x_index = 1:length(x_vec)
    x = x_vec(x_index);
    wb = waitbar((x_index)/length(x_vec),wb);
    %disp([num2str(floor(100 * current_grid_point/num_grid_points)) '%' ])

    %     round(100*(x_index)/length(x_vec))
    for y_index = 1:length(y_vec)
        y = y_vec(y_index);
        for z_index = 1:length(z_vec)
            z = z_vec(z_index);
            
            % GRID
            IJK(current_grid_point,1) = x_index-1;
            IJK(current_grid_point,2) = y_index-1;
            IJK(current_grid_point,3) = z_index-1;
            XYZ(current_grid_point,1) = x;
            XYZ(current_grid_point,2) = y;
            XYZ(current_grid_point,3) = z;
            
            % coordinate conversion from cartesian to spherical
            rc = sqrt(x^2+y^2+z^2);
            theta = atan2(sqrt(x^2+y^2), z);
            phi = atan2(y,x);
            
            % omit calculation outside sphere
            if((rc) > params.a ) % TODO: r=0
                E_field_sph = [0 0 0];
                E_field_x = 0;
                E_field_y = 0;
                E_field_z = 0;
                % calculate inside
            else
                E_field_sph = computeE1atSphCoords(rc, theta, phi, c, d,  params);
                
                % convert to cartesian
                cph = cos(phi);
                sph = sin(phi);
                cth = cos(theta);
                sth = sin(theta);
                E_field_x = E_field_sph(1)*sth*cph + E_field_sph(2)*cth*cph - E_field_sph(3)*sph;
                E_field_y = E_field_sph(1)*sth*sph + E_field_sph(2)*cth*sph + E_field_sph(3)*cph;
                E_field_z = E_field_sph(1)*cth     - E_field_sph(2)*sth;
                
            end
            
            E2_current =  abs(E_field_x)^2 + abs(E_field_y)^2 + abs(E_field_z)^2;
            
            SAR_eval_current = SAR_eval(current_grid_point);
            SAR_mie = E2_current / params.density / 2 * params.sigma;
            SAR_mie_vec(current_grid_point) = SAR_mie;
            
            SAR_error(current_grid_point) = abs(SAR_eval_current - SAR_mie);
            
            % neglect points where SAR is zero (error that might result from meshing inaccuracies)
            if(SAR_eval_current == 0)
                SAR_error(current_grid_point) = 0;
            end
            
            current_grid_point = current_grid_point+1;
            
        end
    end
end
close(wb)
%%
%%
sphere_radius = sqrt(sum(XYZ.^2,2));
% neglect points where SAR is zero (error that might result from meshing inaccuracies)

index_of_interest = SAR_error ~= 0;
index10mm_layer = (sphere_radius > params.a-params.l_unit*10) & (sphere_radius <= params.a) & index_of_interest;
index20mm_layer = (sphere_radius > params.a-params.l_unit*20) & (sphere_radius <= params.a) & index_of_interest;

% Total Sphere
SAR_RMSE_total = sqrt(mean((SAR_mie_vec(index_of_interest) - SAR_eval(index_of_interest)).^2));
SAR_norm_total = sqrt(mean(SAR_mie_vec(index_of_interest).^2));

RMSE_total = 100 * SAR_RMSE_total / SAR_norm_total;

% Outer 10mm
SAR_RMSE_10 = sqrt(mean((SAR_mie_vec(index10mm_layer) - SAR_eval(index10mm_layer)).^2));
SAR_norm_10 = sqrt(mean(SAR_mie_vec(index10mm_layer).^2));

RMSE_10 = 100 * SAR_RMSE_10 / SAR_norm_10;

% Outer 20mm
SAR_RMSE_20 = sqrt(mean((SAR_mie_vec(index20mm_layer) - SAR_eval(index20mm_layer)).^2));
SAR_norm_20 = sqrt(mean(SAR_mie_vec(index20mm_layer).^2));

RMSE_20 = 100 * SAR_RMSE_20 / SAR_norm_20;


