%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
% Calculates total loss, peak SAR and L2-erros for Table 13 of IEC/IEEE 62704-4
% based on the Mie solution, which is an analytic solution of plane-wave        
% scattering on sphere, published in the book:
%
% Craig F. Bohren, Donald R. Huffman, Absorption and scattering of light by small 
% particles, Chapter 4, John Wiley & Sons, 1983
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Initializing
%--------------
clear all; close all; clc
addpath(genpath(pwd));  % add functions

% The sphere has a  
%       radius (params.a) of **100 mm**
%       relative permittivity (params.eps_r) of **41.5**
%       conductivity (params.sigma) of **0.97 S/m**
%       mass density (params.density) of **1000 kg/m3**
% It is irradiated with a linear polarized plane wave 
%       at a frequency (params.freq) of **900 MHz**
%       with an an incident E-field amplitude (params.E0) of **100 V/m**

mie_parameters % load parameters from script

%% Calculates total loss of the Mie solution in the sphere via semi-analytic
%% solution.
%   This is an analytic integration in phi and theta directions, and 
%   numerical in r direction. The total power loss obtained is a reference 
%   value in the standard and it is computed by this function. 
%   Thus, the purpose of running the function is only to verify the 
%   Matlab code by comparing the output here and the value in the standard.
%--------------------------------------------------------------------------

Total_loss = calculate_total_loss(params);
disp(['Total loss = ' num2str(Total_loss) ' W']);


%% Finds peak SAR of the Mie solution on a rectilinear grid with 2mm grid size
%   This brut-force search for the peak is done on a rectilinear grid with 
%   2mmm grid size. This peak SAR reference value is also available in the 
%   standard and the value was calculated by this function. This function could
%   also be run to verify the Matlab code. However, the function is commented 
%   out because its run time is around 20 minutes and the total loss 
%   calculation is sufficient to verify the code.
%------------------------------------------------------------------------------
% Updates resolution grid size to 2 mm 

% params.resolution = 2*params.l_unit;
% params.x_grid = -params.a:params.resolution:params.a;   
% params.y_grid = -params.a:params.resolution:params.a;
% params.z_grid = -params.a:params.resolution:params.a;
% 
% % calculate peak SAR
% peak_SAR = calculate_peak_SAR(params);
% disp(['Peak SAR = ' num2str(peak_SAR) ' W/kg']);


%% Generates and saves rectilinear grid points for L2-error computation
%   The rectilinear grid is generated fully in a cube that tightly includes
%   the sphere. For the error computation we use a **4mm** grid size to 
%   reduce the number of points where the SAR has to be computed in the
%   numerical codes. The file format is defined in Annex C.
%-------------------------------------------------------------------------
% Updates resolution grid size to 4 mm 
params.resolution = 4*params.l_unit;
params.x_grid = -params.a:params.resolution:params.a;   
params.y_grid = -params.a:params.resolution:params.a;
params.z_grid = -params.a:params.resolution:params.a;

% Generates and save grid points to file -
gridFile = 'test_data/MieSphere3DGrid.txt';

saveGrid(params,gridFile);


%% Imports the SAR values at the grid points and computes the relative  
%  L2-error compared to the analitic Mie solution. 
%  Run time is around 90 seconds.
%---------------------------------------------------------------------
% Reads the file with computed SAR data and computes normalized L2-errors.
%   The file utilized below is an example computed by a FE code. The 
%   results of another numerical code can be treated in the same way
%   as long as the file format follows the format in Annex C. The computed 
%   SAR values outside the sphere have to be zero because that is how
%   the error computation function knows what points have to be excluded
%   from the error computation in function compute_L2_SAR_error(). 
%   (Due to the - mostly tetrahedral - discretization, it is not
%   enough to exclude r>a points because the surface of the discretized
%   sphere is constructed by planar triangles, except in the case of
%   curvilinear elements.)

% Updates resolution grid size to 4 mm 
params.resolution = 4*params.l_unit;
params.x_grid = -params.a:params.resolution:params.a;   
params.y_grid = -params.a:params.resolution:params.a;
params.z_grid = -params.a:params.resolution:params.a;

% Generate errors
evalFile = 'test_data/MieSphere3DGrid_localSAR_100Vperm.txt';
[RMSE_total, RMSE_10, RMSE_20] = compute_L2_SAR_error(params,evalFile);

disp(['L2 SAR error for total sphere = ' num2str(RMSE_total) '%'] )
disp(['L2 SAR error for outer layer (10mm) = ' num2str(RMSE_10) '%'] )
disp(['L2 SAR error for outer layer (20mm) = ' num2str(RMSE_20) '%'] )

