%--------------------------------------------------------------------------
%
% Generate the magnetic field (H) errors on the gridfor Layered Patch Test
%
%   INPUT: ABC contains the complex H field in the formnat define 
%          in Annex C; indexes (i,j,k) start at 0
%   OUTPUT: rmsErr(1:3): rms errors for the three field components(Hx,Hy,Hz)
%           maxErr(1:3): maximum errors for the three field components
%           oriErr(1:3): error at the origin
%
%           All E field errors are normalized to E0/eta0 . (The
%           modulus of the incident H field.)
%
%--------------------------------------------------------------------------

function [ rmsErr, maxErr, oriErr ] = computeHfieldErrorInLayeredCube( params, ABC )

siz = size(ABC);
M = siz(1);

H0 = params.E0 / params.eta0;

% calculate point errors of H components
err = zeros(M,3);
mOri = 0;
minR = 1E30;
for m = 1:M
    % get calculated field
    x = ABC(m,4);
    y = ABC(m,5);
    z = ABC(m,6);
    Hx = complex(ABC(m,7), ABC(m,8));
    Hy = complex(ABC(m,9), ABC(m,10));
    Hz = complex(ABC(m,11), ABC(m,12));
    
    % compute incident field
    exactHx = computeExactHx_Layered(params, y);
    exactHy = complex(0,0);
    exactHz = complex(0,0);

    % compute point error
    err(m,1) = abs(Hx-exactHx)/H0;
    err(m,2) = abs(Hy-exactHy)/H0;
    err(m,3) = abs(Hz-exactHz)/H0;
    
    % store origin index
    R = sqrt(x^2 + y^2 + z^2);
    if R < minR 
        minR = R;
        mOri = m;
    end
end

% calculate rms error
rmsErr = [0; 0; 0];
for m = 1:M
    for c=1:3
        rmsErr(c) = rmsErr(c) + err(m,c)^2;
    end
end
for c=1:3
    rmsErr(c) = sqrt( rmsErr(c)/M );
end

% calculate maximum error and the location of maximum error in Ez
maxErr = [0; 0; 0];
maxLoc = [0; 0; 0];
for m = 1:M
    if err(m,1) > maxErr(1)
        maxErr(1) = err(m,1);
    end
    if err(m,2) > maxErr(2)
        maxErr(2) = err(m,2);
    end
    if err(m,3) > maxErr(3)
        maxErr(3) = err(m,3);
        % store location
        maxLoc(1) = ABC(m,4);
        maxLoc(2) = ABC(m,5);
        maxLoc(3) = ABC(m,6);
    end
end

% calculate error at origin
oriErr = [0; 0; 0];
if mOri ~= 0
    for c=1:3
        oriErr(c) = err(mOri,c);
    end
end

end

