clear all

% Reference Result: replace this by the reference files for 1g or 10g and
% graded or uniform mesh
str_ref_file = 'sar_star_graded_ref_1g_2016_03_18';

% Result under test: replace this by the output file of your SAR algorithm
str_test_file = 'sar_result_under_test.txt';

% Log file for the test: replace the file name if needed
str_logFile = 'log_file.txt';

% maximum number of error messages to display per test
int_maxdisp = 10;


% Do not change anything below this line!

global id_LogFile = fopen(str_logFile, 'w');

function LogOut(s)
    fprintf("%s\n", s);
    global id_LogFile;
    fprintf(id_LogFile, "%s\n", s);
end    

function sarDest = ProcessReference(sarOrig, sarTest)
    n_lines = length(sarOrig);
    sarDest = zeros(length(sarTest), 9);
    %
    % Check for multiple entries with different 
    % averaging directions. If a multiple entry
    % is found, its psSAR value is copied to the
    % previous line in a new column. The line
    % with the multiple entry is removed.
    %
    LogOut(sprintf('%d lines in reference, %d in test result', n_lines, length(sarTest)));
    iAmbg = 0;
    sarDest(1, :) = sarOrig(1, :); % copy first line, then start loop with 2nd
    iOrig = 2;
    iDest = 2;
    fprintf('...progress................................................................100%%\n');
    nProgress = int32(n_lines/80);
    while (iOrig <= n_lines)
        if (mod(iOrig,nProgress)==0)
            fprintf('#'); % print progress bar
        end
        
        % Test if the previous line has the same voxel 
        % indices as the current one.
        if (sarOrig(iOrig, 1) == sarDest(iDest - 1, 1)) ...
            && (sarOrig(iOrig, 2) == sarDest(iDest - 1, 2)) ...
            && (sarOrig(iOrig, 3) == sarDest(iDest - 1, 3))    
            
            % If the psSAR value of the current line is larger than 
            % the one of the previous line, override the one of the 
            % previous line.
            if sarOrig(iOrig, 9) > sarDest(iDest - 1, 9)
                sarDest(iDest - 1, 9) = sarOrig(iOrig, 9);
            end
            
            % if orientation is the same as in the test result
            if sarOrig(iOrig, 7)==sarTest(iDest - 1, 7)
                sarDest(iDest - 1, 7) = sarOrig(iOrig, 7); % copy orientation
                sarDest(iDest - 1, 6) = sarOrig(iOrig, 6); % also copy volume
                sarDest(iDest - 1, 5) = sarOrig(iOrig, 5); % and mass
            end
            
            % Make sure that the averaging status of both lines is 
            % "Unused" (index = 1)
            if (abs(sarDest(iDest - 1, 4)) ~= 1) || (sarOrig(iOrig, 4) ~= 1)
                error(['Wrong averaging status for multiple entry in at indices ', ...
                    num2str(sarOrig(iOrig, 1)), ', ', num2str(sarOrig(iOrig, 2)), ...
                    ', ', num2str(sarOrig(iOrig, 3)), ' (i, j, k).'])
            end
            
            % mark that line as ambiguous by negative status flag
            sarDest(iDest - 1, 4) = -sarOrig(iOrig, 4); 
            
            iOrig = iOrig + 1;
        else
            sarDest(iDest, :) = sarOrig(iOrig, :);
            iOrig = iOrig + 1;
            iDest = iDest + 1;
        end
        
    end
    fprintf('\n')
    LogOut(sprintf('%d lines in processed reference', iDest-1));
end
clc
more off
format long % high precision needed to see differences in volume and mass
disp('')
LogOut('IEC/IEEE 62704-1 SAR Star Evaluation Script')
LogOut('Version 1.1')
LogOut('July 31st, 2016')
disp('')
disp('Status, mass and volume are tested for all voxels. The spatial')
disp('average SAR is only tested for UNUSED voxels.') 
disp('')
disp('Implemented and tested in GNU Octave version 4.00')
disp('')

list_str_quantities = {'Status', 'Mass', 'Volume', 'Spatial Average SAR'};
list_str_units = {'', 'g', 'mm^3', 'W/kg'};
list_int_columns = [4, 5, 6, 9];
list_flt_tolerances = [0, 2.e-6, 2.e-6, 0.2];
boo_alltestspassed = 1;
LogOut(['Loading "',str_test_file,'" ...']); 
tabTest = load(str_test_file);
LogOut(['Loading "',str_ref_file,'" ...']); 
tabOrig = load(str_ref_file);
tabRef = ProcessReference(tabOrig, tabTest);
if numel(tabTest) ~= numel(tabRef) 
    error('The number of array elements in the input files does not agree.')
end
disp('')
disp('Beginning validation...')
disp('')

for i = 1:length(list_str_quantities) 
    i_linesInTable = (1:length(tabRef));
    
    % The averaging mass and volume are not defined for 'used' voxels.
    % This is why their values are disregarded.
    if strcmp(list_str_quantities{i}, 'Mass') || strcmp(list_str_quantities{i}, 'Volume')
        i_linesInTable = find(tabRef(:, 4) ~= 2);
    
    % Test the spatial average SAR in ambiguous 'unused' voxels only.
    elseif strcmp(list_str_quantities{i}, 'Spatial Average SAR')
        i_linesInTable = find(tabRef(:, 4) == -1);
    end

    LogOut(sprintf("%s test (%d lines in table relevant):", ...
        list_str_quantities{i}, length(i_linesInTable)));
    if strcmp(list_str_quantities{i}, 'Status')
        % compare integer values (i. e., status)
        list_deviations = tabTest(i_linesInTable, list_int_columns(i)) - ...
                       abs(tabRef(i_linesInTable, list_int_columns(i)));
    else
        % compare floats with their list_flt_tolerances
        list_deviations = tabTest(i_linesInTable, list_int_columns(i)) ./ ...
                           tabRef(i_linesInTable, list_int_columns(i)) - 1;
    end
    
    % find and output maximum deviation if relevant
    [MaxErr, int_MaxErr] = max(abs(list_deviations));
    if int_MaxErr~=0
        if abs(list_deviations(int_MaxErr)) > 0.01*list_flt_tolerances(i)
            id = i_linesInTable(int_MaxErr);
            LogOut(sprintf('Maximum deviation at (i/j/k) %d %d %d:', ...
                tabTest(id,1), tabTest(id,2), tabTest(id,3)));
            LogOut(sprintf('    Reference: %.12g %s  Test: %.12g %s', ...
                tabRef(id, list_int_columns(i)), list_str_units{i}, ...
                tabTest(id, list_int_columns(i)), list_str_units{i}));
            if list_flt_tolerances(i) ~= 0
                LogOut(sprintf('    Deviation is %f%%, Tolerance is +/-%f%%.', ...
                    100 * list_deviations(int_MaxErr), ...
                    100 * list_flt_tolerances(i)));
            end
        end
    end

    if strcmp(list_str_quantities{i}, 'Spatial Average SAR')
        list_int_errors = find(list_deviations < -list_flt_tolerances(i));
    else
        list_int_errors = find(abs(list_deviations) > list_flt_tolerances(i));
    end
 
    int_nErrors = length(list_int_errors);
    LogOut(sprintf('%d Deviations >%f percent', int_nErrors, list_flt_tolerances(i)));
    for ierr = 1:min(int_nErrors, int_maxdisp)
        id = i_linesInTable(list_int_errors(ierr));
        LogOut(sprintf('Error at (i/j/k) %d %d %d:', ...
            tabTest(id,1), tabTest(id,2), tabTest(id,3)));
        LogOut(sprintf('    Reference: %.12g %s  Test: %.12g %s', ...
            tabRef(id, list_int_columns(i)), list_str_units{i}, ...
            tabTest(id, list_int_columns(i)), list_str_units{i}));
       	if list_flt_tolerances(i) ~= 0
            LogOut(sprintf('    Deviation is %f%%, Tolerance is +/-%f%%.', ...
                100 * list_deviations(list_int_errors(ierr)), ...
                100 * list_flt_tolerances(i)));
        end
    end
 
    if int_nErrors == 0
        LogOut('*** PASSED ***')
    else
        boo_alltestspassed = 0;
        if int_nErrors - int_maxdisp > 0
            LogOut(sprintf('     ... and %d more error(s).', int_nErrors - int_maxdisp));
        end
        LogOut('*** FAILED ***');
    end
    
    LogOut('');
    
end
if boo_alltestspassed == 1
    LogOut('*** ALL TESTS WERE PASSED. ***')
else
    LogOut('*** ONE OR MORE TESTS FAILED. ***')
end
fclose(id_LogFile);
